/* PRO-SCAMS "@@(#) init_dial.c, Rev: 1.6.0.0" */

/* -----                                                                */
/*                                                                      */
/* UWDIALER Version 1.0                                        12/09/93 */
/*                                                                      */
/* AUTHOR: William C. Brown                                             */
/* VERSION: 1.0 (beta)                                                  */
/* DESCRIPTION: This little section of code sets up the dial() function */
/*		and then sets the line terminal disciplines.		*/ 
/*                                                                      */
/* -------------------------------------------------------------------- */
/* Notes:                                                               */
/*                                                                      */
/*                                                                ----- */

#include <stdio.h>
#include <termio.h>
#include <sys/types.h>
#include <sys/signal.h>
#include <dial.h>
#include <stropts.h>
#include <errno.h>
#include <cs.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <Xol/OpenLook.h>
#include <Xol/ScrollingL.h>

#include "uwdialer.h"

int Modem_fd;
int pipe_fd[2];
int Server_Pid = 0;
CALL call;
XtInputId Pipe_Output;
void Reap_Dialer_Child();

void Init_Dial(Dial_Info *Dfb, int Ind)
{
	char *Ret;
	char Temp[BUFSIZ];
	char Proto_Buf[10];
        Arg args[2];
	struct sigaction sig;
	extern Widget Te1, Te2;
	extern XtAppContext app_con;
	extern int Lock_Pid;

	(void)memset(&call, NULL, sizeof(CALL));
	(void)Set_Buttons(True);

	/* -----					*/
	/* Get the System information.			*/
	/*					  ----- */

	if(Ind == MANUAL) {
		XtSetArg(args[0], XtNstring, &Dfb->Line);
		XtGetValues(Te1, args, 1);

		XtSetArg(args[0], XtNstring, &Dfb->System);
		XtGetValues(Te2, args, 1);
	}
	else
		Dfb->Line = NULL;

	call.attr  = NULL;
	call.baud  = NULL;
	call.speed = Dfb->Speed;
	call.line  = Dfb->Line;
	call.telno = Dfb->System;

	if(strlen(Dfb->System) != 0) 
		(void)sprintf(Temp, "Dialing %s", Dfb->System);
	else if(strlen(Dfb->Line) != 0)
		(void)sprintf(Temp, "Direct connect %s", Dfb->Line);
	else {
		(void)DialerError(NONFATAL, MISSINGSYSTEM);
		return;
	}

	(void)Display_Msg(False, Temp);

	/* -----					*/
	/* Call the remote system.			*/
	/*					  ----- */


	if(pipe(pipe_fd) != 0) {
		(void)DialerError(NONFATAL, OPENPIPEFAILED);
		return;
	}

	/* -----					*/
	/* Run the dial function as a separate process  */
	/* so that X doesn't have to wait for it. This  */
	/* keeps our screens nice an fresh. Return the  */
	/* FD through a streams pipe.			*/
	/*					  ----- */

	if((Server_Pid = fork()) == 0) {
		(void)sleep(1);
		(void)close(pipe_fd[0]);
		Modem_fd = dial(call);

		(void)strcpy(Proto_Buf, "Positive");
		if(Modem_fd < 0) {
			Ret = Resolve_Dial_Error(Modem_fd);
			(void)DialerError(NONFATAL, DIALFAILS, errno, Ret);
			(void)strcpy(Proto_Buf, "Negative");
		}
		else {
			(void)strcpy(Proto_Buf, "Positive");
		}

		if(write(pipe_fd[1], Proto_Buf, 8) != 8) {
			(void)DialerError(NONFATAL, PROTOBUFWRFAIL);
			(void)exit(0);
		}

		if(Modem_fd >= 0) {
			if(ioctl(pipe_fd[1], I_SENDFD, Modem_fd) < 0) {
				(void)DialerError(NONFATAL, PROTOIOCTL);
				(void)exit(0);
			}
		}
		exit(0);
	} /* fork */

	Lock_Pid = Server_Pid;

	sig.sa_handler = Reap_Dialer_Child;
        sig.sa_flags = 0;
        sig.sa_flags |= SA_RESTART;
        sigaction(SIGCHLD, &sig, NULL);

	Pipe_Output = XtAppAddInput(app_con, pipe_fd[0], XtInputReadMask,
			Complete_Connection, Dfb);

} /* Init_Dial */

void Chars_At_Modem(XtPointer ClientData, int *Src, XtInputId *inid)
{
        char buffer[BUFSIZ];
	char *ptr;
        int nbytes;
	extern FILE *Logger_fp;
	extern int LOGGER;
	extern int Xfer_Direction;

	if(Xfer_Direction != 0)
		return;

	if(LOGGER == ON)
		(void)memset(buffer, NULL, BUFSIZ);

        if((nbytes = read(Modem_fd, buffer, BUFSIZ)) < 0) {
		if(errno == EINTR)
			return;
		(void)DialerError(NONFATAL, ZEROREAD);
	}
        else
                (void)write(fileno(stdout), buffer, nbytes);

	if(LOGGER == ON) {
		ptr = Strip(buffer);
		(void)write(fileno(Logger_fp), ptr, strlen(ptr));
	}

} /* Chars_At_Modem */

char *Resolve_Dial_Error(int Modem_fd)
{
	static char Error[BUFSIZ];

	switch (Modem_fd) {
		case -1:
			(void)strcpy(Error, "interrupt occurred");
			return(Error);
		case -2:
			(void)strcpy(Error, "dialer failed");
			return(Error);
		case -3:
			(void)strcpy(Error, "no answer");
			return(Error);
		case -4:
			(void)strcpy(Error, "illegal baud rate");
			return(Error);
		case -5:
			(void)strcpy(Error, "acu problem");
			return(Error);
		case -6:
			(void)strcpy(Error, "line problem (open failed)");
			return(Error);
		case -7:
			(void)strcpy(Error, "can't open devices file");
			return(Error);
		case -8:
			(void)strcpy(Error, "requested device not available");
			return(Error);
		case -9:
			(void)strcpy(Error, "requested device not known");
			return(Error);
		case -10:
			(void)strcpy(Error, "no devices available at baud rate");
			return(Error);
		case -11:
			(void)strcpy(Error, "no devices known at baud rate");
			return(Error);
		case -12:
			(void)strcpy(Error, "system not in system file");
			return(Error);
		case -13:
			(void)strcpy(Error, "no connection server");
			return(Error);
		case -14:
			return(Error);
		default:
			(void)strcpy(Error, "unknown error");
			return(Error);
	} /* case */
} /* Resolve_Dial_Error */

int Report_Speed(struct termios *t)
{
	if(cfgetospeed(t) == B300) {
		return(300);
	}
	if(cfgetospeed(t) == B1200) {
		return(1200);
	}
	if(cfgetospeed(t) == B2400) {
		return(2400);
	}
	if(cfgetospeed(t) == B4800) {
		return(4800);
	}
	if(cfgetospeed(t) == B9600) {
		return(9600);
	}
	if(cfgetospeed(t) == B19200) {
		return(19200);
	}
	if(cfgetospeed(t) == B38400) {
		return(38400);
	}
	return(0);
} /* Report_Speed */

char *Strip(char *string)
{
	static char Temp[BUFSIZ];
	char *ptr1, *ptr2;

	(void)memset(Temp, NULL, BUFSIZ);

	for(ptr1=string,ptr2=Temp;*ptr1!=NULL;ptr1++) {
		if(*ptr1 == RETURN)
			continue;
		else {
			*ptr2++ = *ptr1;
		}
	}

	return(Temp);
} /* Strip */

void Complete_Connection(Dial_Info *Dfb, int *Src, XtInputId *Inid)
{
	char buf[BUFSIZ];
	char Temp[BUFSIZ];
	struct termios t;
	struct strrecvfd recfd;
	extern XtInputId Modem_Input;
	extern XtAppContext app_con;
	extern int Lock_Pid;

	/* -----					*/
	/* Rewrite the lock file as soon as possible so */
	/* that uucp or cu can't steal our tty.		*/
	/*					  ----- */

	(void)Delete_Lock(REWRITE);
	Lock_Pid = getpid();

	(void)memset(buf, NULL, BUFSIZ);
	XtRemoveInput(Pipe_Output);
	Server_Pid = 0;

	if(read(pipe_fd[0], buf, 8) < 8) {
		(void)DialerError(NONFATAL, SERVERTERM); 
		(void)strcpy(buf, "Negative");
	}

	if(strncmp(buf, "Positive", 8) == 0) {
		if(ioctl(pipe_fd[0], I_RECVFD, &recfd) < 0) {
			(void)DialerError(NONFATAL, PROTOIOCTL);
			return;
		}
		Modem_fd = recfd.fd;
	} 
	else {
		Modem_fd = -1;
	}

        if(Modem_fd < 0) {
		(void)Set_Buttons(False);
                (void)Call_Hangup(NULL, NULL, NULL);
                (void)Call_Repaint_Display(NULL, NULL, NULL);
                return;
        }

	(void)tcgetattr(Modem_fd, &t);

        t.c_oflag = 0;
        t.c_iflag =  IGNBRK | ISTRIP | IGNPAR | Dfb->Flow;
        t.c_cflag |= CREAD | HUPCL | CLOCAL | Dfb->CharSize | Dfb->Parity;
	t.c_lflag = 0;

	switch (Dfb->Speed) {
		case 300:
			cfsetispeed(&t, B300);
			cfsetospeed(&t, B300);
			break;
		case 1200:
			cfsetispeed(&t, B1200);
			cfsetospeed(&t, B1200);
			break;
		case 2400:
			cfsetispeed(&t, B2400);
			cfsetospeed(&t, B2400);
			break;
		case 4800:
			cfsetispeed(&t, B4800);
			cfsetospeed(&t, B4800);
			break;
		case 9600:
			cfsetispeed(&t, B9600);
			cfsetospeed(&t, B9600);
			break;
		case 19200:
			cfsetispeed(&t, B19200);
			cfsetospeed(&t, B19200);
			break;
		case 38400:
			cfsetispeed(&t, B38400);
			cfsetospeed(&t, B38400);
			break;
	} /* switch */

	if(strlen(call.telno) != 0) {
		(void)sprintf(Temp, "Connected to %s - %d",
			 Dfb->System, Report_Speed(&t));
	}
	else if(strlen(call.line) != 0)
		(void)sprintf(Temp, "Direct to %s", Dfb->Line);

        (void)tcsetattr(Modem_fd, TCSANOW, &t);

	(void)Set_Buttons(False);
	(void)Set_System_Buttons(True);
	(void)Call_Repaint_Display(NULL, NULL, NULL);
	(void)Update_Connect_Clock((XtPointer)START, NULL);
	(void)Display_Msg(False, Temp);

    	Modem_Input = XtAppAddInput(app_con, Modem_fd, XtInputReadMask,
			 Chars_At_Modem, NULL);

	(void)close(pipe_fd[0]);
	(void)close(pipe_fd[1]);

} /* Complete_Connection */

void Reap_Dialer_Child()
{
	int stat;

	(void)wait(&stat);
} /* Reap_Dial_Child */
