/* PRO-SCAMS "@@(#) uwdialer.c, Rev: 1.4.0.0" */

/* -----								*/
/* 									*/
/* UWDIALER Version 1.0					       12/05/93 */
/*									*/
/* AUTHOR: William C. Brown						*/
/* VERSION: 1.0 (beta)							*/
/* DESCRIPTION: To provide a nice X based dialer interface that works	*/
/*		in conjunction with the connection server in SVR4.2.	*/
/*									*/
/* -------------------------------------------------------------------- */
/* Notes: 								*/
/*									*/
/*								  ----- */

#include <stdio.h>
#include <stdlib.h>
#include <stropts.h>
#include <termio.h>
#include <pwd.h>
#include <grp.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <Xol/OpenLook.h>
#include <Xol/RubberTile.h>
#include <Xol/ControlAre.h>
#include <Xol/Stub.h>
#include <Xol/StaticText.h>
#include <Xol/FButtons.h>
#include <Xol/ScrollingL.h>

#include "uwdialer.h"

/* Widgets */

Widget  TopLevel,
	Rubber,
	BaseControl,
	Control1,
	Flats,
	Count,
	ConStub,
	Stub;

/* Some extra globals */

GC ORgc, XORgc;
XFontStruct *Lft, *Sft, *Tft;
XtAppContext app_con;
XtIntervalId TimerId;
XGCValues gcv;

int Uid, Gid, UUCPgid;
int Xterm_Type = TERMINAL;
int Xtermid;
int LOGGER = OFF;
int Xfer_Direction = 0;
int Lock_Pid = 0;
char *ProgName;
struct termios old;
Dial_Info DialInfoBuf;
Pixel RedPixel, GreenPixel;
Boolean Xfer_InProgress = False;

XtInputId Term_Input;
XtInputId Modem_Input;

void Call_Enter(Widget, XtPointer, XEvent *, Boolean *);
void Get_Dialer_Options(int, char **);
void Usage();

/* FlatItems stuff */

static String	But_Fields[] = {XtNlabel, XtNselectProc, XtNbusy};
static XtArgVal But_Items[] = 
	{(XtArgVal)"Quit",        (XtArgVal)Call_Quit,    (XtArgVal)False,
         (XtArgVal)"Systems",     (XtArgVal)Call_Systems, (XtArgVal)False,
	 (XtArgVal)"Manual Dial", (XtArgVal)Call_Manual,  (XtArgVal)False,
	 (XtArgVal)"Hangup",      (XtArgVal)Call_Hangup,  (XtArgVal)False,
	 (XtArgVal)"Upload",      (XtArgVal)Call_Upload,  (XtArgVal)False,
	 (XtArgVal)"Download",    (XtArgVal)Call_Downld,  (XtArgVal)False,
	 (XtArgVal)"Send Break",  (XtArgVal)Call_SndBrk,  (XtArgVal)False,
	 (XtArgVal)"Log",         (XtArgVal)Call_Log,     (XtArgVal)False};

void main(int argc, char *argv[])
{
	int AC = 0;
	int X, Y, XtermWidth, FlatWidth, height, b, d;
	unsigned int Num_Cld;
	struct group *grp, *getgrnam();
	static char *Large_Font = "-*-courier-*-*-*-*-24-*-*-*-*-*-*";
	static char *Small_Font = "-*-courier-*-*-*-*-20-*-*-*-*-*-*";
	static char *Tiny_Font = "-*-lucidatypewriter-*-*-*-*-*-80-*-*-*-*-*-*";
	char *L_Font;
	char *S_Font;
	char *T_Font;

	char *Icon_Name;
	Boolean argok;
	Arg args[10];
	Window RootWin, Parent, GParent, *Children;
	XtTranslations Translation;
	Pixmap Prog_Pixmap;
	XrmValue from, to;

	ProgName = argv[0];
	grp = getgrnam("uucp");
	UUCPgid = grp->gr_gid;
	(void)Init_Personal_file();
	Uid = getuid();
	Gid = getgid();
	(void)setgid(UUCPgid);
	(void)memset(&DialInfoBuf, NULL, sizeof(Dial_Info));

	OlToolkitInitialize(&argc, argv, (XtPointer)NULL);

	(void)Get_Dialer_Options(argc, argv);

	XtSetArg(args[AC], XtNresizeCorners, FALSE);           AC++;
	XtSetArg(args[AC], XtNtitle, "UWDIALER Version 1.0");  AC++;
	TopLevel = XtAppInitialize(&app_con, "uwdialer", "UWdialer", 0,
				   &argc, argv, NULL, args, AC);

	/* -----					*/
	/* Get the pixmap name from the Xresource db    */
	/* and convert that into something we can use.	*/
	/*					  ----- */

	Icon_Name = XGetDefault(XtDisplay(TopLevel), argv[0], "iconPixmap");

	from.addr = Icon_Name;
	from.size = strlen(Icon_Name)+1;
	to.addr   = (char *)&Prog_Pixmap;
	to.size   = sizeof(Pixmap);

	if(XtConvertAndStore(TopLevel, XtRString, &from,
			     XtRPixmap, &to) != False) {
		XtSetArg(args[0], XtNiconPixmap, Prog_Pixmap);
		XtSetValues(TopLevel, args, 1);
	}

	/* -----				*/
	/* Get the fonts names out of the       */
	/* resource file. If not there default  */
	/* to some hand selected fonts.		*/
	/*				  ----- */

	L_Font = XGetDefault(XtDisplay(TopLevel), argv[0], "largeFont");
	S_Font = XGetDefault(XtDisplay(TopLevel), argv[0], "smallFont");
	T_Font = XGetDefault(XtDisplay(TopLevel), argv[0], "tinyFont");

	if(L_Font == NULL)
		L_Font = Large_Font;

	if(S_Font == NULL)
		S_Font = Small_Font;

	if(T_Font == NULL)
		T_Font = Tiny_Font;

	/* -----				*/
	/* Create a base level rubber tile.	*/
	/*				  ----- */

	AC = 0;
	XtSetArg(args[AC], XtNshadowThickness, 2);             AC++;
	XtSetArg(args[AC], XtNshadowType, OL_SHADOW_OUT);      AC++;
	XtSetArg(args[AC], XtNpaneGravity, NorthSouthGravity); AC++;
	Rubber = XtCreateManagedWidget ("rubber", rubberTileWidgetClass,
					TopLevel, args, AC);

	/* -----				*/
	/* Now a control area for the buttons   */
	/* and such, then a stub widget that we */
	/* will use to reparent the Xterm.	*/
	/*				  ----- */

	AC = 0;
	XtSetArg(args[AC], XtNlayoutType, OL_FIXEDROWS);  AC++;
	XtSetArg(args[AC], XtNmeasure, 1); 		  AC++;
	XtSetArg(args[AC], XtNshadowThickness, 0);        AC++;
	XtSetArg(args[AC], XtNshadowType, OL_SHADOW_IN); AC++;
	Control1 = XtCreateManagedWidget ("control", controlAreaWidgetClass,
					Rubber, args, AC);

	AC = 0;
	XtSetArg(args[AC], XtNbuttonType, OL_OBLONG_BTN); 	    AC++;
	XtSetArg(args[AC], XtNitemFields, But_Fields);    	    AC++;
	XtSetArg(args[AC], XtNnumItemFields, XtNumber(But_Fields)); AC++;
	XtSetArg(args[AC], XtNitems, But_Items);                    AC++;
	XtSetArg(args[AC], XtNlayoutType, OL_FIXEDCOLS);            AC++;
	XtSetArg(args[AC], XtNmeasure, 4);                          AC++;
	XtSetArg(args[AC], XtNnumItems, MAIN_BUTTONS);              AC++;
	Flats = XtCreateManagedWidget("button", flatButtonsWidgetClass,
				     Control1, args, AC);
 
	AC = 0;
	XtSetArg(args[AC], XtNlayoutType, OL_FIXEDROWS);  AC++;
	XtSetArg(args[AC], XtNmeasure, 1); 		  AC++;
	XtSetArg(args[AC], XtNshadowThickness, 1);        AC++;
	XtSetArg(args[AC], XtNshadowType, OL_SHADOW_IN);  AC++;
	Count = XtCreateManagedWidget("count", controlAreaWidgetClass,
					Control1, args, AC);

	AC = 0;
	XtSetArg(args[AC], XtNheight, 40);                       AC++;
	XtSetArg(args[AC], XtNwidth, 10);                        AC++;
	XtSetArg(args[AC], XtNbackground,
		 CONVERT(Count, "black", "Pixel", 0, &argok));   AC++;
	XtSetArg(args[AC], XtNborderWidth, 1);                   AC++;
	XtSetArg(args[AC], XtNborderColor,
		 CONVERT(Count, "green", "Pixel", 0, &argok));   AC++;
	XtSetArg(args[AC], XtNexpose, Call_Repaint_Display);     AC++;
	ConStub = XtCreateManagedWidget ("stub", stubWidgetClass,
					Count, args, AC);

	AC = 0;
	XtSetArg(args[AC], XtNheight, 10);                AC++;
	XtSetArg(args[AC], XtNwidth, 10);                 AC++;
	XtSetArg(args[AC], XtNweight, 10);                AC++;
	Stub = XtCreateManagedWidget ("stub", stubWidgetClass,
					Rubber, args, AC);

	/* -----					*/
	/* Realize the widget tree, then do some size   */
	/* calculations.				*/
	/*					  ----- */

	XtRealizeWidget (TopLevel);

	Xtermid = atoi(getenv("WINDOWID"));
	XGetGeometry(XtDisplay(Stub), Xtermid, &RootWin, &X, &Y,
		     &XtermWidth, &height, &b, &d);
 
	if(Xterm_Type == TERMINAL) {
		XQueryTree(XtDisplay(Stub), Xtermid, &RootWin,
			   &Parent, &Children, &Num_Cld);
		XQueryTree(XtDisplay(Stub), Parent, &RootWin,
			   &GParent, &Children, &Num_Cld);
		XReparentWindow(XtDisplay(Stub), GParent, XtWindow(Stub), 0, 0);
	}
	else if (Xterm_Type == XTERM) {
    		XReparentWindow(XtDisplay(Stub), Xtermid, XtWindow(Stub), 0, 0);
	}
	
	XtAddEventHandler(Stub, EnterWindowMask, True, Call_Enter, Xtermid);

	AC = 0;
	XtSetArg(args[AC], XtNheight, height);                AC++;
	if(Xterm_Type == TERMINAL) {
		XtSetArg(args[AC], XtNwidth, XtermWidth+20);  AC++;
	}
	else {
		XtSetArg(args[AC], XtNwidth, XtermWidth);     AC++;
	}
	XtSetValues(Stub, args, AC);

	XGetGeometry(XtDisplay(Stub), XtWindow(Flats), &RootWin, &X, &Y,
		     &FlatWidth, &height, &b, &d);

	if(Xterm_Type == XTERM)
		FlatWidth = FlatWidth + 20;

	AC = 0;
	XtSetArg(args[AC], XtNwidth, XtermWidth-FlatWidth); AC++; 
	XtSetValues(ConStub, args, AC);

	/* -----						*/
	/* Get a GC for use with the status display.		*/
	/*						  ----- */

	RedPixel       = GetPixelByName(ConStub, "red");
	GreenPixel     = GetPixelByName(ConStub, "green");
	gcv.foreground = GreenPixel;
	gcv.background = GetPixelByName(ConStub, "black");
	gcv.function   = GXxor;

	if ((Lft = XLoadQueryFont(XtDisplay(Stub), L_Font)) != NULL)
		gcv.font = Lft->fid; 
	else
		(void)DialerError(FATAL, FONTALLOCATION, Large_Font);

	if ((Sft = XLoadQueryFont(XtDisplay(Stub), S_Font)) == NULL)
		(void)DialerError(FATAL, FONTALLOCATION, Small_Font);

	if ((Tft = XLoadQueryFont(XtDisplay(Stub), T_Font)) == NULL)
		(void)DialerError(FATAL, FONTALLOCATION, Tiny_Font);

	XORgc = XCreateGC(XtDisplay(Stub), XtWindow(ConStub),
			GCForeground | GCBackground | GCFont | GCFunction,
			&gcv);

	gcv.function   = GXor;
	ORgc = XCreateGC(XtDisplay(Stub), XtWindow(ConStub),
			GCForeground | GCBackground | GCFont | GCFunction,
			&gcv);

	/* -----						*/
	/* Get everything initialized.				*/
	/*						  ----- */

	(void)Update_Connect_Clock((XtPointer)INIT, NULL);
	(void)Display_Msg(True, "Initializing...");
	(void)Init_Xterm();
	(void)Display_Msg(False, "Ready...");

	XtAppMainLoop(app_con);

} /* main() */

void Call_Enter(Widget W, XtPointer ClientData, XEvent *event, Boolean *Ctn)
{
	XSetInputFocus(XtDisplay(Stub), ClientData, RevertToNone, 0);
}

void Get_Dialer_Options(int argc, char *argv[])
{
	int c;
	char *terminal_type;
	extern char *optarg;
	extern int optind;

	while ((c = getopt(argc, argv, "t:")) != EOF) {
		switch(c) {
			case 't':
				terminal_type = optarg;
				break;
			case '?':
				(void)Usage();
		}
	}

	if(strcmp(terminal_type, "terminal") == 0)
		Xterm_Type = TERMINAL;
	else if(strcmp(terminal_type, "xterm") == 0)
		Xterm_Type = XTERM;
	else
		(void)Usage();
} /* Get_Dialer_Options */

void Usage()
{
	fprintf(stderr, "USAGE: [xterm_program] -e uwdialer -t xterm_type\n");
	(void)exit(0);
} /* Usage */

void Init_Personal_file()
{
	char Path[BUFSIZ];
	char Create_Command[BUFSIZ];
	struct passwd *pwd, *getpwuid();

	pwd = getpwuid(getuid());
	(void)sprintf(Path, "%s/%s", pwd->pw_dir, PERSONAL_FILE);

	if(access(Path, F_OK) == -1) {
		(void)sprintf(Create_Command, ">%s;chmod 660 %s;chgrp uucp %s",
			Path, Path, Path);
		(void)system(Create_Command);
	}
} /* Init_Personal_File */
